@@ -1,3 +1,8 @@
+"""
+Tests for bitarray
+
+Author: Ilan Schnell
+"""
 import os
 import sys
 import unittest
@@ -121,7 +126,7 @@ tests.append(TestsModuleFunctions)
 # ---------------------------------------------------------------------------
     
 class CreateObjectTests(unittest.TestCase, Util):
-    
+
     def test_noInitializer(self):
         a = bitarray()
         self.assertEqual(len(a), 0)
@@ -129,21 +134,25 @@ class CreateObjectTests(unittest.TestCase, Util):
         self.check_obj(a)
 
     def test_endian(self):
-        a = bitarray('A', 0, endian='little')
+        a = bitarray(endian='little')
+        a.fromstring('A')
         self.assertEqual(a.endian(), 'little')
         self.check_obj(a)
 
-        b = bitarray('A', 0, endian='big')
+        b = bitarray(endian='big')
+        b.fromstring('A')
         self.assertEqual(b.endian(), 'big')
         self.check_obj(b)
 
         self.assertEqual(a.tostring(), b.tostring())
         
-        a = bitarray(' ', 0, endian=u'little')
+        a = bitarray(endian=u'little')
+        a.fromstring(' ')
         self.assertEqual(a.endian(), 'little')
         self.check_obj(a)
 
-        b = bitarray(' ', 0, endian=u'big')
+        b = bitarray(endian=u'big')
+        b.fromstring(' ')
         self.assertEqual(b.endian(), 'big')
         self.check_obj(b)
         
@@ -161,7 +170,7 @@ class CreateObjectTests(unittest.TestCase, Util):
             a = bitarray(long(n))
             self.assertEqual(len(a), n)
             self.check_obj(a)
-            
+        
         self.assertRaises(ValueError, bitarray.__new__, bitarray, -1)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, -924L)
 
@@ -282,7 +291,7 @@ class CreateObjectTests(unittest.TestCase, Util):
         
         self.assertRaises(TypeError, bitarray.__new__, bitarray, '', 0, 42)
         self.assertRaises(ValueError, bitarray.__new__, bitarray, '', 0, 'foo')
-        
+
         
 tests.append(CreateObjectTests)
 
@@ -361,7 +370,7 @@ class SliceTests(unittest.TestCase, Util):
         self.assertRaises(IndexError, a.__getitem__,  7)
         self.assertRaises(IndexError, a.__getitem__, -8)
         
-        a = bitarray('AQ', 3)
+        a = bitarray('0100000100001')
         self.assertEQUAL(a[:], a)
         self.assert_(a[:] is not a)
         aa = a.tolist()
@@ -668,12 +677,17 @@ class NumberTests(unittest.TestCase, Util):
         self.assertEQUAL(c, bitarray('001110'))
         
         for a in self.randombitarrays():
+            aa = a.copy()
             for b in self.randombitarrays():
+                bb = b.copy()
                 c = a + b
                 self.assertEqual(c, bitarray(a.tolist() + b.tolist()))
                 self.assertEqual(c.endian(), a.endian())
                 self.check_obj(c)
                 
+                self.assertEQUAL(a, aa)
+                self.assertEQUAL(b, bb)
+                
         a = bitarray()
         self.assertRaises(TypeError, a.__add__, 42)
 
@@ -709,6 +723,7 @@ class NumberTests(unittest.TestCase, Util):
         self.assertEQUAL(c, bitarray('110110110'))
         
         for a in self.randombitarrays():
+            b = a.copy()
             for n in xrange(-10, 20):
                 c = a * n
                 self.assertEQUAL(c, bitarray(n * a.tolist(),
@@ -717,14 +732,19 @@ class NumberTests(unittest.TestCase, Util):
                 self.assertEqual(c, bitarray(n * a.tolist(),
                                              endian=a.endian()))
 
+                self.assertEQUAL(a, b)
+                
+
         a = bitarray()
         self.assertRaises(TypeError, a.__mul__, None)
     
 
     def test_imul(self):
         c = bitarray('1101110011')
+        idc = id(c)
         c *= 0
         self.assertEQUAL(c, bitarray())
+        self.assertEqual(idc, id(c))
         
         c = bitarray('110')
         c *= 3
@@ -732,12 +752,12 @@ class NumberTests(unittest.TestCase, Util):
         
         for a in self.randombitarrays():
             for n in xrange(-10, 10):
-                b = bitarray(a)
-                c = b
-                c *= n
-                self.assertEQUAL(c, bitarray(n * a.tolist(),
+                b = a.copy()
+                idb = id(b)
+                b *= n
+                self.assertEQUAL(b, bitarray(n * a.tolist(),
                                              endian=a.endian()))
-                self.assert_(c is b)
+                self.assertEqual(idb, id(b))
         
         a = bitarray()
         self.assertRaises(TypeError, a.__imul__, None)
@@ -766,13 +786,17 @@ class BitwiseTests(unittest.TestCase, Util):
         self.assertEQUAL(a & b, bitarray('10001'))
 
         b = bitarray('1001')
-        self.assertRaises(ValueError, a.__and__, b)
+        self.assertRaises(ValueError, a.__and__, b) # not same length
+
+        self.assertRaises(TypeError, a.__and__, 42)
 
         
     def test_iand(self):
         a =  bitarray('110010110')
+        ida = id(a)
         a &= bitarray('100110011')
         self.assertEQUAL(a, bitarray('100010010'))
+        self.assertEqual(ida, id(a))
 
     def test_or(self):
         a = bitarray('11001')
@@ -1135,22 +1159,6 @@ class MethodTests(unittest.TestCase, Util):
             self.assertEQUAL(a, bitarray(aa[::-1], endian=a.endian()))
 
 
-    def test_fromlist(self):
-        a = bitarray()
-        a.fromlist([]);         self.assertEQUAL(a, bitarray())
-        a.fromlist([1]);        self.assertEQUAL(a, bitarray('1'))
-        a.fromlist([0, 1, 1]);  self.assertEQUAL(a, bitarray('1011'))
-        
-        for a in self.randombitarrays():
-            for blst in self.randomlists():
-                alst = a.tolist()
-                c = a
-                c.fromlist(blst)
-                self.assertEqual(c.tolist(), alst + blst)
-                self.assert_(c is a)
-                self.check_obj(c)
-                
-        
     def test_tolist(self):
         a = bitarray()
         self.assertEqual(a.tolist(), [])
@@ -1243,7 +1251,8 @@ class MethodTests(unittest.TestCase, Util):
         self.assertEqual(a, bitarray('10110111'))
         
         for i in xrange(256):
-            a = bitarray(chr(i), 0)
+            a = bitarray()
+            a.fromstring(chr(i)) 
             aa = a.tolist()
             b = a
             b.bytereverse()
@@ -1264,13 +1273,13 @@ class StringTests(unittest.TestCase, Util):
 
 
     def test_fromstring(self):
-        a = bitarray()
+        a = bitarray(endian='big')
         a.fromstring('A')
-        self.assertEqual(a, bitarray('A', 0))
+        self.assertEqual(a, bitarray('01000001'))
         
         b = a
         b.fromstring('BC')
-        self.assertEQUAL(b, bitarray('ABC', 0))
+        self.assertEQUAL(b, bitarray('01000001' '01000010' '01000011'))
         self.assert_(b is a)
         
         for b in self.randombitarrays():
@@ -1280,7 +1289,8 @@ class StringTests(unittest.TestCase, Util):
         
         for b in self.randombitarrays():
             for s in self.randomstrings():
-                a = bitarray(s, 0, endian=b.endian())
+                a = bitarray(endian=b.endian())
+                a.fromstring(s)
                 c = b.__copy__()
                 b.fromstring(s)
                 self.assertEQUAL(b[-len(a):], a)
@@ -1293,7 +1303,8 @@ class StringTests(unittest.TestCase, Util):
         self.assertEqual(a.tostring(), '')
         
         for end in ('big', 'little'):
-            a = bitarray('foo', 0, endian=end)
+            a = bitarray(endian=end)
+            a.fromstring('foo')
             self.assertEqual(a.tostring(), "foo")
 
             for s in self.randomstrings():
@@ -1310,24 +1321,6 @@ class StringTests(unittest.TestCase, Util):
             self.assertEqual(a.tostring(), s)
 
 
-    def test_tofrom01(self):
-        for a in self.randombitarrays():
-            b = bitarray()
-            c = b
-            c.from01(a.to01())
-            self.assertEqual(a, c)
-            self.assert_(b is c)
-            self.check_obj(c)
-            
-        a = bitarray()
-        self.assertRaises(ValueError, a.from01, '01201')
-        
-        a = bitarray()
-        a.from01('10');         self.assertEqual(a.to01(), '10')
-        a.from01('');           self.assertEqual(a.to01(), '10')
-        a.from01('01110');      self.assertEqual(a.to01(), '1001110')
-        a.from01('100');        self.assertEqual(a.to01(), '1001110100')
-
     def test_unpack(self):
         a = bitarray('01')
         self.assertEqual(a.unpack(), '\x00\xff')
@@ -1354,6 +1347,7 @@ class StringTests(unittest.TestCase, Util):
             b.invert()
             self.assertEqual(b, a)
 
+
     def test_pack(self):
         a = bitarray()
         a.pack('\x00')
@@ -1420,6 +1414,21 @@ class FileTests(unittest.TestCase, Util):
         self.assertEqual(a, bitarray())
 
 
+    def test_from_large_file(self):
+        N = 100000
+
+        fo = open(self.tmpfname, 'wb')
+        fo.write(N * 'X')
+        fo.close()
+
+        a = bitarray()
+        a.fromfile(open(self.tmpfname, 'rb'))
+        self.assertEqual(len(a), 8 * N)
+        self.assertEqual(a.buffer_info()[1], N)
+        # make sure there is no over-allocation
+        self.assertEqual(a.buffer_info()[4], N)
+
+
     def test_fromfile_Foo(self):
         fo = open(self.tmpfname, 'wb')
         fo.write('Foo\n')
@@ -1447,7 +1456,8 @@ class FileTests(unittest.TestCase, Util):
 
 
     def test_fromfile_n(self):
-        a = bitarray('ABCDEFGHIJ', 0)
+        a = bitarray()
+        a.fromstring('ABCDEFGHIJ')
         fo = open(self.tmpfname, 'wb')
         a.tofile(fo)
         fo.close()
@@ -1507,8 +1517,8 @@ class FileTests(unittest.TestCase, Util):
             s = open(self.tmpfname, 'rb').read()
             self.assertEqual(len(s), a.buffer_info()[1])
 
-        for n in xrange(10):
-            a = bitarray(n * 'A', 0)
+        for n in xrange(3):
+            a.fromstring(n * 'A')
             self.assertRaises(TypeError, a.tofile)
             self.assertRaises(TypeError, a.tofile, StringIO())
             
@@ -1528,7 +1538,10 @@ class FileTests(unittest.TestCase, Util):
             
             b = a.__copy__()
             b.fill()
-            self.assertEqual(bitarray(s, 0, endian='little'), b)
+
+            c = bitarray(endian='little')
+            c.fromstring(s)
+            self.assertEqual(c, b)
 
 
 tests.append(FileTests)
@@ -1697,47 +1710,49 @@ tests.append(PrefixCodeTests)
 # ---------------------------------------------------------------------------
 
 def pages():
+    if sys.platform != 'linux2':
+        return 0
+    
     dat = open('/proc/%i/statm' % os.getpid()).read()
     return int(dat.split()[0])
 
-def run(verbosity, chk_mem_leaks=False, times=1):
+
+def check_memory_leaks(verbosity):
+    suite = unittest.TestSuite()
+    for cls in tests[:-1]:  # [ CreateObjectTests ]:
+        suite.addTest(unittest.makeSuite(cls))
+    
+    logfile = 'pages.log'
+    try: os.unlink(logfile)
+    except: pass
+    i = 0
+    runner = unittest.TextTestRunner(verbosity=verbosity)
+    while True:
+        print 'Run', i
+        r = runner.run(suite)
+        if i % 1 == 0:
+            fo = open(logfile, 'a')
+            fo.write('%10i %r %10i\n' % (i, r.wasSuccessful(), pages()))
+            fo.close()
+        i += 1
+
+
+def run(verbosity, chk_mem_leaks=False):
+    if chk_mem_leaks:
+        check_memory_leaks(verbosity)
+    
     suite = unittest.TestSuite()
     for cls in tests:
         suite.addTest(unittest.makeSuite(cls))
     
     runner = unittest.TextTestRunner(verbosity=verbosity)
-    for dum in xrange(times):
-        res = runner.run(suite)
-
-    if chk_mem_leaks:
-        if sys.platform == 'linux2':
-            i = 0
-            logfile = 'pages.log'
-            try: os.unlink(logfile)
-            except: pass
-            while True:
-                r = runner.run(suite)
-                p = pages()
-                if i % 100 == 0:
-                    fo = open(logfile, 'a')
-                    fo.write('%10i %r %10i\n' % (i, r.wasSuccessful(), p))
-                    fo.close()
-                i += 1
-        else:
-            print 'Sorry, can only test for memory leaks on Linux.'
-
-    return res
+    
+    return runner.run(suite)
 
 
 if __name__ == '__main__':
-    arg = ''
-    if len(sys.argv)==2:
-        arg = sys.argv[1]
-    
-    run(verbosity     = 2 if 'v' in arg else 1,
-        chk_mem_leaks = bool('m' in arg),
-        times         = int(arg) if arg.isdigit() else 1
-        )
+    run(verbosity     = 2 if 'v' in sys.argv else 1,
+        chk_mem_leaks = bool('m' in sys.argv))
 
 else:
     from bitarray import __version__
